@@ -283,6 +283,8 @@ class Collection(object):
     # Column you can't query
     NON_QUERYABLE_COLUMNS = ["_json"]
     DEFAULT_COLUMNS = ["_id", "_json", "_created_at", "_modified_at"]
+    _columns = []
+    _indexes = []
 
     def __init__(self, db, name):
         self.name = name
@@ -311,12 +313,13 @@ class Collection(object):
         Returns:
             list
         """
-        with self.db:
-            try:
-                _exec = self.db.execute("SELECT * FROM %s LIMIT 1" % self.name)
-                return _exec.fetchall()[0].keys()
-            except: pass
-        return []
+        if not self._columns:
+            with self.db:
+                try:
+                    _exec = self.db.execute("PRAGMA table_info(%s)" % self.name)
+                    self._columns = [r["name"] for r in _exec.fetchall()]
+                except: pass
+        return self._columns
 
     @property
     def indexes(self) -> list:
@@ -326,12 +329,20 @@ class Collection(object):
         Returns:
             list
         """
-        with self.db:
-            try:
-                _exec = self.db.execute("SELECT * FROM sqlite_master WHERE type=?", ("index",))
-                return [item["tbl_name"] for item in _exec]
-            except: pass
-        return []
+        if not self._indexes:
+            with self.db:
+                try:
+                    indexes = []
+                    _exec = self.db.execute("PRAGMA index_list(%s)" % self.name)
+                    for item in _exec.fetchall():
+                        name = item["name"]
+                        _i2 = self.db.execute("PRAGMA index_info(%s)" % name)
+                        f = _i2.fetchone()
+                        indexes.append(f["name"])
+                    self._indexes = indexes
+                    #return [item.keys() for item in _exec]
+                except: pass
+        return self._indexes
 
     @property
     def size(self) -> int:
@@ -398,7 +409,6 @@ class Collection(object):
         query += " LIMIT 1"
 
         xparams = list(filters.values())
-
         with self.db:
             _exec = self.db.execute(query, xparams)
             row = _exec.fetchone()
@@ -585,69 +595,89 @@ class Collection(object):
         with self.db:
             self.db.execute("DROP TABLE %s " % self.name)
 
-    def ensure_columns(self, columns:list):
+    def add_columns(self, columns:list):
         """
         To add columns 
-
         Args:
             colums: list
                 [
                     (column, type),
-                    (column, type)
+                    (column, type),
+                    (column, 'text', 'UNIQUE'|True)
                 ]
 
         Returns:
             None
         """
         with self.db:
-            _columns = self.columns
             for _ in columns:
-                col, coltype = _
-
-                if col in _columns: # column exist already
-                    continue
-
                 try:
-                    # attempt to add column
-                    q = "ALTER TABLE %s ADD COLUMN %s %s DEFAULT NULL" % (self.name, col, coltype)
-                    self.db.execute(q)
-                except:
-                    pass
+                    colindex = None
+                    if len(_) == 3:
+                        col, coltype, colindex = _
+                    else:
+                        col, coltype = _
 
-    def ensure_indexes(self, indexes:list):
+                    # Add column if not exists
+                    if col not in self.columns: 
+                        self.db.execute("ALTER TABLE %s ADD COLUMN %s %s DEFAULT NULL" % (self.name, col, coltype))
+
+                    # Add index if not exists
+                    if colindex and col not in self.indexes:
+                        index_stmt = None
+                        # unique
+                        if isinstance(colindex, str) and colindex.upper() == "UNIQUE":
+                            index_stmt = "UNIQUE"
+                        # regular
+                        elif colindex is True:
+                            index_stmt = ""
+                        if index_stmt is not None:
+                            self.db.execute("CREATE %s INDEX %s ON %s (%s)" % (index_stmt, col+"__idx", self.name, col))
+                except Exception as e: pass
+        # reset columns
+        self._columns = []
+        self._indexes = []
+
+    def add_indexes(self, indexes:list):
         """
-        To add indexes
+        Add indexes ensuring columns
 
         Args:
-            indexes: list
-                [
-                    (column, type),
-                    (column, type)
-                ]
+            indexes: list of indexes. 
+
+            :format:
+            [
+                column:str # index the column
+                (column) # index the column
+                (column, "UNIQUE") # index column as unique
+                (column, True) # index column as unique
+                (column, TYPE=TEXT|INT...) index column with type 
+                (column, TYPE=TEXT|INT..., "UNIQUE") index column with type and unique
+            ]
 
         Returns:
             None
         """
-        with self.db:
-            columns = self.columns
-            _indexes = self.indexes
-            
-            for _ in indexes:
-                col, coltype = _
+        new_indexes = []
+        for idx in indexes:
+            if isinstance(idx, str):
+                new_indexes.append((idx, 'TEXT', True))
+            elif isinstance(idx, (list, set, tuple)):
+                if len(idx) == 1: # colum
+                    new_indexes.append((idx[0], 'TEXT', True))
+                elif len(idx) == 2: # (column, columntype)
+                    _type = idx[1]
+                    # Unique index - (column, ':UNIQUE:') or (column, True)
+                    if _type == "UNIQUE" or _type is True: 
+                        new_indexes.append((idx[0], 'TEXT', "UNIQUE"))
+                    # (column, 'TEXT|INT|...')
+                    else:
+                        new_indexes.append((idx[0], _type, True))
+                elif len(idx) == 3: # (column, type, UNIQUE)
+                    new_indexes.append((idx[0], idx[1], idx[2]))
+        if new_indexes:
+            self.add_columns(new_indexes)
 
-                if col in columns: # column exist already
-                    continue
-                try:
-                    # attempt to add column
-                    if col not in columns:
-                        q = "ALTER TABLE %s ADD COLUMN %s %s DEFAULT NULL" % (self.name, col, coltype)
-                        self.db.execute(q)
-                    
-                    # attempt to add index
-                    if col not in _indexes: 
-                        self.db.execute("CREATE INDEX %s ON %s (%s)" % (col+"_index", self.name, col))
-                except:
-                    pass
 
     def __len__(self):
         return self.size
